declare type State = {
    [key: string]: any;
};
declare type Apply<T> = (value: T) => T;
declare type ArgumentApply<T, A> = (argument: A) => Apply<T>;
declare type StandaloneMutation = Apply<State> & {
    standalone: true;
};
declare type ArgumentMutation<A> = ArgumentApply<State, A> & {
    standalone: false;
};
export declare const makeStandaloneMutation: <T extends any>(apply: Apply<T>) => (field: string | number) => StandaloneMutation;
export declare const makeStandaloneHook: <T extends any>(apply: Apply<T>, defaultValue: T) => (initialValue?: T) => (T | (() => void))[];
export declare const makeArgumentMutation: <T extends any, A extends any>(apply: ArgumentApply<T, A>) => (field: string | number) => ArgumentMutation<A>;
export declare const makeArgumentHook: <T extends any, A extends any>(apply: ArgumentApply<T, A>, defaultValue: T) => (initialValue?: T) => (T | ((argument: any) => void))[];
export declare const decrementState: (value: number) => number;
export declare const decrement: (field: string | number) => StandaloneMutation;
export declare const useDecrement: (initialValue?: number) => (number | (() => void))[];
export declare const incrementState: (value: number) => number;
export declare const increment: (field: string | number) => StandaloneMutation;
export declare const useIncrement: (initialValue?: number) => (number | (() => void))[];
export declare const toggleState: (value: boolean) => boolean;
export declare const toggle: (field: string | number) => StandaloneMutation;
export declare const useToggle: (initialValue?: boolean) => (boolean | (() => void))[];
export declare const appendState: <T extends any>(argument: T) => (value: T[]) => T[];
export declare const append: <T extends any>(field: string | number) => ArgumentMutation<T>;
export declare const useAppend: (initialValue?: any[]) => (any[] | ((argument: any) => void))[];
export declare const concatState: <T extends any>(argument: T | T[]) => (value: T[]) => T[];
export declare const concat: <T extends any>(field: string | number) => ArgumentMutation<T | T[]>;
export declare const useConcat: (initialValue?: any[]) => (any[] | ((argument: any) => void))[];
export declare const cycleState: <T extends any>(argument: T[]) => (value: T) => T;
export declare const cycle: <T extends any>(field: string | number) => ArgumentMutation<T[]>;
export declare const useCycle: <T extends any>(argument: T[]) => [T, (...args: any[]) => any];
export declare const directState: <T extends any>(argument: T) => () => T;
export declare const direct: <T extends any>(field: string | number) => ArgumentMutation<T>;
export declare const filterState: <T extends any>(argument: (value: T) => boolean) => (value: T[]) => T[];
export declare const filter: <T extends any>(field: string | number) => ArgumentMutation<(value: T) => boolean>;
export declare const useFilter: (initialValue?: any[]) => (any[] | ((argument: any) => void))[];
export declare const mapState: <T extends any>(argument: (value: T) => T) => (value: T[]) => T[];
export declare const map: <T extends any>(field: string | number) => ArgumentMutation<(value: T) => T>;
export declare const useMap: (initialValue?: any[]) => (any[] | ((argument: any) => void))[];
export declare const mutateState: (argument: object) => (value: object) => {};
export declare const mutate: (field: string | number) => ArgumentMutation<object>;
export declare const prependState: <T extends any>(argument: T) => (value: T[]) => T[];
export declare const prepend: <T extends any>(field: string | number) => ArgumentMutation<T>;
export declare const usePrepend: (initialValue?: any[]) => (any[] | ((argument: any) => void))[];
export declare const combineMutations: (...mutations: (State | StandaloneMutation | ArgumentMutation<any>)[]) => Apply<State>;
export {};
