"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
exports.makeStandaloneMutation = function (apply) { return (function (field) {
    var mutation = function (state) {
        var _a;
        return (_a = {}, _a[field] = apply(state[field]), _a);
    };
    mutation.standalone = true;
    return mutation;
}); };
exports.makeStandaloneHook = (function (apply, defaultValue) { return function (initialValue) {
    if (initialValue === void 0) { initialValue = defaultValue; }
    var _a = react_1.useState(initialValue), value = _a[0], setValue = _a[1];
    var onMutate = react_1.useCallback(function () { return setValue(apply); }, [setValue]);
    return [value, onMutate];
}; });
exports.makeArgumentMutation = function (apply) { return (function (field) {
    var mutation = function (argument) {
        var curried = apply(argument);
        return function (state) {
            var _a;
            return (_a = {}, _a[field] = curried(state[field]), _a);
        };
    };
    mutation.standalone = false;
    return mutation;
}); };
exports.makeArgumentHook = (function (apply, defaultValue) { return (function (initialValue) {
    if (initialValue === void 0) { initialValue = defaultValue; }
    var _a = react_1.useState(initialValue), value = _a[0], setValue = _a[1];
    var onMutate = react_1.useCallback(function (argument) { return setValue(apply(argument)); }, [
        setValue
    ]);
    return [value, onMutate];
}); });
exports.decrementState = function (value) { return value - 1; };
exports.decrement = exports.makeStandaloneMutation(exports.decrementState);
exports.useDecrement = exports.makeStandaloneHook(exports.decrementState, 0);
exports.incrementState = function (value) { return value + 1; };
exports.increment = exports.makeStandaloneMutation(exports.incrementState);
exports.useIncrement = exports.makeStandaloneHook(exports.incrementState, 0);
exports.toggleState = function (value) { return !value; };
exports.toggle = exports.makeStandaloneMutation(exports.toggleState);
exports.useToggle = exports.makeStandaloneHook(exports.toggleState, true);
exports.appendState = function (argument) { return function (value) { return (value.concat(argument)); }; };
exports.append = exports.makeArgumentMutation(exports.appendState);
exports.useAppend = exports.makeArgumentHook(exports.appendState, []);
exports.concatState = function (argument) { return function (value) { return (value.concat(argument)); }; };
exports.concat = exports.makeArgumentMutation(exports.concatState);
exports.useConcat = exports.makeArgumentHook(exports.concatState, []);
exports.cycleState = function (argument) { return function (value) { return (argument[(argument.indexOf(value) + 1) % argument.length]); }; };
exports.cycle = exports.makeArgumentMutation(exports.cycleState);
exports.useCycle = function (argument) {
    var _a = react_1.useState(argument[0]), value = _a[0], setValue = _a[1];
    var cycleValue = react_1.useCallback(exports.cycleState(argument), [argument]);
    var onCycle = react_1.useCallback(function () { return setValue(cycleValue); }, [cycleValue]);
    return [value, onCycle];
};
exports.directState = function (argument) { return function () { return argument; }; };
exports.direct = exports.makeArgumentMutation(exports.directState);
exports.filterState = function (argument) { return function (value) { return (value.filter(argument)); }; };
exports.filter = exports.makeArgumentMutation(exports.filterState);
exports.useFilter = exports.makeArgumentHook(exports.filterState, []);
exports.mapState = function (argument) { return function (value) { return (value.map(argument)); }; };
exports.map = exports.makeArgumentMutation(exports.mapState);
exports.useMap = exports.makeArgumentHook(exports.mapState, []);
exports.mutateState = function (argument) { return function (value) { return (__assign(__assign({}, value), argument)); }; };
exports.mutate = exports.makeArgumentMutation(exports.mutateState);
exports.prependState = function (argument) { return function (value) { return ([argument].concat(value)); }; };
exports.prepend = exports.makeArgumentMutation(exports.prependState);
exports.usePrepend = exports.makeArgumentHook(exports.prependState, []);
var normalizeMutation = function (mutation) {
    if (typeof mutation !== "object") {
        return mutation;
    }
    var directMutation = function () { return mutation; };
    directMutation.standalone = true;
    return directMutation;
};
exports.combineMutations = function () {
    var mutations = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        mutations[_i] = arguments[_i];
    }
    var normalized = mutations.map(normalizeMutation);
    if (normalized.some(function (mutation) { return !mutation.standalone; })) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return function (state) {
                var argIndex = -1;
                return normalized.reduce(function (nextState, mutation) {
                    var apply;
                    if (mutation.standalone) {
                        apply = mutation;
                    }
                    else {
                        argIndex += 1;
                        apply = mutation(args[argIndex]);
                    }
                    return __assign(__assign({}, nextState), apply(nextState));
                }, state);
            };
        };
    }
    return function (state) { return normalized.reduce(function (nextState, mutation) { return (__assign(__assign({}, nextState), mutation(nextState))); }, state); };
};
